### Назначение пакета.

### Описание.

### Формат backend файлов.

backend файлы имеют суффикс _.backend_ и хранятся в следующих директориях:
/usr/share/alterator/backends/user/ и /etc/alterator/backends/user/ - для
пользовательского режима (--user),
/usr/share/alterator/backends/, /usr/share/alterator/backends/system/ и
/etc/alterator/backends/, /etc/alterator/backends/system/ для обычного режима.
Директории названы в том порядке, в котором они обрабатываются, т.е. сначала
считываются файлы из /usr/share/alterator/backends/, затем из
/usr/share/alterator/backends/system/, затем из /etc/alterator/backends/, и
потом из /etc/alterator/backends/system/. Для пользовательского режима сначала
из /usr/share/alterator/backends/user/, потом из /etc/alterator/backends/user/.
Один backend файл описывает один интерфейс некоторого объекта. Поскольку на
одном объекте не может быть интерфейсов с одинаковыми именами, то 'дубликаты'
игнорируются при загрузке.
Формат backend файлов следующий:

```
[Alterator Entry]
Type = Backend
Module = executor
Name = first2
Interface = second
#Interface = ru.basealt.alterator.second //Интерфейс можно задать полным именем.
thread_limit = 5
action_id = org.freedesktop.test


[make_ls2]
execute = ls -al {param}

[make_ls3]
execute = ls -al {param}
```

Первая секция *[Alterator Entry]* обязательная, она имеет следующие пары
ключ-значение:
1. Type - зависит от назначения файла. Пока заполняется словом _Backend_.
1. Module - имя модуля который будет обрабатывать запросы.
1. Name - имя узла (объекта) в поддереве D-Bus.
1. Interface - имя интерфейса D-Bus.
1. thread_limit - максимальное число тредов для данного интерфейса.
1. action_id - action id для polkit.

Если имя интерфейса задано одной секцией (т.е. без точек, _second_ в примере
выше), то к _interface_name_ автоматически добавляется префикс
_'ru.basealt.alterator.'_. Если оно состоит из нескольких секций (разделённых
точками, например _ru.basealt.alterator.second_), то префикс не добавляется.
Пустые секции не допускаются. Каждая секция может содержать только
[A-Z][a-z][0-9]_.

Опция actiont_id не обязательная. Это actiont id для методов интерфейса
(polkit). Если actiont id не указан, то он формируется из имени интерфейса, для
чего подчёркивания заменяются на тире.

Опция thread_limit не обязательная. Если её явно не задать, то её значение
будет равно 10 (максимальное число методов выполняемых одновременно).

Каждая следующая секция - это метод. Имя секции - имя метода.
Имя метода может состоять только из латинских букв, цифр и символа
подчёркивания. Пробелы не допускаются.
Содержимое секции зависит от модуля который будет обрабатывать запрос.
Выше приведён пример для модуля _executor_. Поле _execute_ содержит строку для
_bash_ с исполняемым файлом. Внутри фигурных скобок могут задаваться параметры
для метода D-Bus. Параметров может не быть. Имя параметра может содержать только
латинские буквы, цифры и символ подчёркивания. В примере выше метод _make_ls2_
будет выполнять команду _ls -al_ с параметром _param_, в который можно будет
передать имя директории или файла.

Описание формата для методов других модулей будет описано здесь, либо в
документации этих модулей.

### Хранение данных.

Данные полученные от модулей и считанные из _backend_ файлов alterator-manager
хранит в хеш-таблице (backends_data). Ключами в ней являются имена узлов в
дереве D-Bus, а значениями хеш-таблицы (interfaces). Ключами в interface'ах
являются имена интерфейсов, а значениями - структуры _InterfaceObjectInfo_.
```
typedef struct {
    gchar *module_name;
    gchar *action_id;
    GDBusInterfaceInfo *interface_introspection;
    GHashTable *methods;
    const GDBusInterfaceVTable *interface_vtable;
    gint thread_limit;
} InterfaceObjectInfo;
```

module_name - имя модуля.
action_id - action id для интерфейса (для polkit).
interface_introspection - GDBusInterfaceInfo из модуля.
thread_limit - лимит тредов для интерфейса.
interface_vtable - GDBusInterfaceVTable из модуля.
methods - хеш-таблица с методами. Ключами являются имена методов (имена секций в
_backend_), а значениями - хеш-таблицы с информацией о методах, взятой из
_backend_ файлов. В этих таблицах ключи - это ключи из секции данного метода,
а значения - это занчения соответствующих ключей. Например, для _executor'а_
ключом будет _execute_, а значением - исполняемая строка.


                    backends_data
        key: node_name             value: interfaces
                                            |
                                            |
                                            |
                          -------------------
                          ||
                          ||
                          \/
                      interface
        key: interface_name        value: InterfaceObjectInfo->methods
                                                                 |
                                                                 |
                                                                 |
                          ----------------------------------------
                          ||
                          ||
                          \/
                        method
        key: method_name           value: method_data
                                            |
                                            |
                                            |
                          -------------------
                          ||
                          ||
                          \/
                     method_data
    key: it depends on the module. value: it depends on the module
    For example - executor:
    key: execute                   value: executable file

### Валидация интерфейсов.

Валидация интерфейсов - это проверка соответствия интерфейса сформированного из
файла с расширением _.backend_ шаблону. Шаблон - это файл интроспекции
проверяемого интерфейса, имя этого файла состоит из имени интерфейса и
расширения _.xml_. Файлы-шаблоны лежат в /usr/share/dbus-1/interfaces.
Валидация считается не пройденной если для интерфейса существует шаблон, но не
все методы описанные в шаблоне описаны в проверяемом интерфейсе, или не
совпадают описания методов.

### Переменные окружения.

В корневом объекте, в интерфейсе ru.basealt.alterator.manager, есть методы
с помощью которых можно добавлять и удалять переменные окружения. Эти переменные
хранятся в хеш-таблице sender_environment_data, она устроена следующим образом:

                      sender_environment_data
               key: sender                value: variables
                                                     |
                                                     |
                                                     |
                                ----------------------
                                ||
                                ||
                                \/
                             variable
               key: name                  value: value

_sender_ - это уникальное имя на D-Bus (например :1.951). Пользователь
подключается к D-Bus, получает уникальное имя и добавляет переменные окружения.
В этот момент в таблице sender_environment_data создаётся пара
sender->variables, где _variables_ это тоже хеш-таблица в которой и хранятся
переменные в виде пар _name_->_value_. Далее, если пользователь (sender)
запускает новый процесс с помощью модуля _executor_, то набор переменных
окружения унаследованный этим процессом от предка (alterator-manager)
дополняется набором из хеш-таблицы для этого пользователя (sender'а).

Раз в 10 минут происходит проверка. Уникальные имена (sender'ы) которых нет
среди подключённых к D-Bus удаляются из таблицы sender_environment_data.
Удаляются и связанные с ними переменные.

### am-dev-tool

am-dev-tool - утилита из пакета alterator-manager-tools, она предназначена для
генерации и валидации _policy_ файлов.

Пример генерации с сохранением результата в файле:
$am-dev-tool -g /usr/share/alterator/backends/file.backend -o ~/file.policy

Пример валидации:
$am-dev-tool -t /usr/share/alterator/backends/file.backend

Опция -q подавляет вывод.

### Модули.

## executor.

```
[method_name]
execute = ls -al {param}
stdout_strings = enabled
stdout_bytes = enabled
stderr_strings = enabled
stdout_signal_name = stdout_signal_example
stderr_signal_name = stderr_signal_example
stdout_byte_limit = 200000
stdout_strings_limit = 200000
stderr_strings_limit = 200000
thread_limit = 3
action_id = method-name
timeout = 60
```

Выше приведены возможные опции метода для модуля _executor_. Поле _execute_ -
обязательное, остальные - нет.
Поле _execute_ содержит строку для _bash_ с исполняемым файлом. Внутри фигурных
скобок могут задаваться параметры для метода D-Bus. Параметров может не быть.
Имя параметра может содержать только латинские буквы, цифры и символ
подчёркивания.

С помощью полей stdout_strings, stdout_bytes и stderr_strings можно включить
возможность возврата массива строк или массива байт из stdout и массива строк
из stderr. Для этого полю нужно задать значение _enabled_. Если включен возврат
массива байт, то возврат строк из stdout отключается (в том числе сигналами).
Максимальный размер в байтах для соответствующего массива можно задать с помощью
полей stdout_byte_limit, stdout_strings_limit и stderr_strings_limit. Значение
по умолчанию 524288. Допустимый диапазон - от 0 до 2147483647. Работа с
массивами больше 524288 не проверялась.

Вывод из _stdout_ и _stderr_ можно также получать с помощью сигналов. Имя
сигнала получается путём конкатенации sender'а и значения соответствующего поля
(stdout_signal_name или stderr_signal_name). В sender'е ':' и '.' заменяются на
'_'. Значения полей stdout_signal_name и stderr_signal_name могут содержать
только латинские буквы, цифры и символ подчёркивания. Сигналы для stdout и
stderr включены, если соответствующему полю задано значение.

С помощью поля thread_limit можно задать ограничение на количество тредов для
метода, т.е. сколько экземпляров этого метода можно запустить параллельно.
Значение по умолчанию - 1.

В поле _action_id_ можно задать _action_id_ polkit для метода. Поле может
содержать только следующие символы [A-Z][a-z][0-9].-
К значению этого поля будет автоматически добавлен префикс сформированный
из имени интерфейса.

В поле _timeout_ задаётся промежуток времени по истечении которого процессу,
порождённому для выполнения данного метода, будет отправлен сигнал SIGKILL.
Время задаётся в секундах. Значение по умолчанию - 60 секунд. Если значение
поля _timeout_ ноль или меньше, то timeout не задаётся и сигнал SIGKILL не будет
отправлен. Если значение задано не корректно - timeout не задаётся и сигнал
SIGKILL не будет отправлен.

